//
//  serial.h
//
//  Created by Orly Natan on 21/09/17.
//


#ifndef serial_h
#define serial_h

#include <termios.h>        // required for: termios struct
#include <sys/param.h>      // required for: MAXPATHLEN const
#include <IOKit/IOBSD.h>    // required for: io_iterator_t

static int macos_ports(io_iterator_t  *PortIterator, char **list, int max, int index);

typedef struct {
    int port_is_open;
    int baud_rate;
    int port_fd;
    char port_name[MAXPATHLEN];
    char error_msg[MAXPATHLEN];
    struct termios settings_orig;
    struct termios settings;
} serial_t;

typedef serial_t* serial_ptr;

int serial_isopen(serial_t *port);
int serial_open(serial_t *port, char *name);
const char *serial_getname(serial_t *port);
int serial_setbaud(serial_t *port, int baud);
long serial_read(serial_t *port, void *ptr, int count);
long serial_write(serial_t *port, const void *ptr, int len);
int serial_inputwait(serial_t *port, int msec);
void serial_inputdiscard(serial_t *port);
void serial_outputflush(serial_t *port);
int serial_setcontrol(serial_t *port, int dtr, int rts);
void serial_close(serial_t *port);
int serial_portlist(char **list, int max_ports);

#endif /* serial_h */
