//
//  main.c
//  testFirmataClient
//
//  Created by Orly Natan on 17/10/17.
//  Copyright © 2017 Orly Natan. All rights reserved.
//

#include "maxlib.h"

// thread library
#include <stdio.h>

#include <assert.h>
#include <pthread.h>

#include<stdlib.h>


bool working = false;

void moveServo(board_t *board, int pin, int newVal)
{
    if (pin<0 || pin>127) return;
    
    if (newVal < 0) newVal = 0;
    if (newVal > 170) newVal = 170;
    //int val = (newVal * 180) / 1023;
    
    //printf("moveServo: new value: %d->%d", newVal, val);
    firmataclient_setnewvalue(board, pin, newVal);
}

// connect to board via a serial port, init board to requested configuration
void initConfigPort(char *name, int baud, int len_config, int *pins, int *modes) {
    
    // open connection to the board, and request REPORT_FIRMATA
    bool result = firmataclient_initport(&board, name, baud);
    if (result == false) {
        printf("firmataclient_initport: couldn't open port. Exiting.\n");
        return;
    }
    
    // wait for board to be ready for communication
    firmataclient_waitForBoardReady(&board);
    
    // now that the board is ready for communication
    // set the correct pin configuration as requested (if supported),
    // or default is MODE_OFF (255)
    if(len_config > 128) len_config = 128;
    for(int i=0; i<len_config; ++i) {
        firmataclient_changemode(&board, pins[i], modes[i]);
    }
}

// incomplete
int test1() {
    // fix test1 from email files
    int max_ports = MAX_PORTS;
    char *list[max_ports];
    
    // printout list of availble ports
    int num_ports = serial_portlist(list, max_ports);
    for (int i=0; i<num_ports; ++i)
        printf("(%d). %s\n", i+1, list[i]);
    
    // start firmata client
    char *name = "/dev/cu.usbmodem1411";
    int baud = DEFAUTLT_BAUD;
    
    // open connection to the board, and request REPORT_FIRMATA
    bool result = firmataclient_initport(&board, name, baud);
    if (result == false) {
        printf("firmataclient_initport: couldn't open port. Exiting.\n");
        return 0;
    }
    
    //wait until you get a REPORT_FIRMATA response
    while( !firmataclient_isboardready(&board) ) {
        firmataclient_onidle(&board);
    }
    
    // todo: add set pin modes - got erased by mistake..
    //
    
    
    firmataclient_onidle(&board);
    firmataclient_setdigitalvalue(&board, 11, 1);
    
    for(int16_t i=0; i<180; i+=30) {
        moveServo(&board, 9, i);
        for(int j=0; j<100; ++j)
            firmataclient_onidle(&board);
    }
    moveServo(&board, 9, 0);
    firmataclient_setdigitalvalue(&board, 11, 0);
    
    for(int j=0; j<100; ++j)
        firmataclient_onidle(&board);
    
    // close the port
    firmataclient_closeport(&board);
    
    printf("Done.\n");
    return 0;
}

//
int test2() {
    
    int max_ports = MAX_PORTS;
    char *list[max_ports];
    
    // printout list of availble ports
    int num_ports = serial_portlist(list, max_ports);
    for (int i=0; i<num_ports; ++i)
        printf("(%d). %s\n", i+1, list[i]);
    
    // start firmata client
    char *name = "/dev/cu.usbmodem1411";
    int baud = DEFAUTLT_BAUD;
    
    const int len_pins = 4;
    int pins[] = {9, 11, 14, 18};
    int modes[] = {MODE_SERVO, MODE_OUTPUT, MODE_ANALOG, MODE_ANALOG};
    
    // open connection to the board, set requested configuration
    // and request REPORT_FIRMATA
    initConfigPort(name, baud, len_pins, pins, modes);
    
    // send (servo, led) values on board
    firmataclient_onidle(&board);
    firmataclient_setdigitalvalue(&board, 11, 1);
    
    for(int16_t i=0; i<180; i+=30) {
        moveServo(&board, 9, i);
        for(int j=0; j<100; ++j)
            firmataclient_onidle(&board);
    }
    moveServo(&board, 9, 0);
    firmataclient_setdigitalvalue(&board, 11, 0);
    
    for(int j=0; j<100; ++j)
        firmataclient_onidle(&board);
    
    // close the port
    firmataclient_closeport(&board);
    
    printf("Done.\n");
    return 0;
}


// test thread
void* test3_thread(void *data) {
    
    working = true;
    
    int max_ports = MAX_PORTS;
    char *list[max_ports];
    
    // printout list of availble ports
    int num_ports = serial_portlist(list, max_ports);
    for (int i=0; i<num_ports; ++i)
        printf("(%d). %s\n", i+1, list[i]);
    
    // start firmata client
    char *name = "/dev/cu.usbmodem1411";
    int baud = DEFAUTLT_BAUD;
    
    maxclient_listenerOn();
    
    const int len_pins = 4;
    int pins[] = {9, 11, 14, 18};
    int modes[] = {MODE_SERVO, MODE_OUTPUT, MODE_ANALOG, MODE_ANALOG};
    
    // open connection to the board, set requested configuration
    // and request REPORT_FIRMATA
    initConfigPort(name, baud, len_pins, pins, modes);
    
    // send (servo, led) values on board
    firmataclient_onidle(&board);
    firmataclient_setdigitalvalue(&board, 11, 1);
    
    for(int16_t i=0; i<180; i+=30) {
        moveServo(&board, 9, i);
        for(int j=0; j<100; ++j)
            firmataclient_onidle(&board);
    }
    moveServo(&board, 9, 0);
    firmataclient_setdigitalvalue(&board, 11, 0);
    
    for(int j=0; j<100; ++j)
        firmataclient_onidle(&board);
    
    // close the port
    firmataclient_closeport(&board);
    
    printf("Done.\n");
    
    working = false;
    
    return NULL;
}

// threading
void test3()
{
    // Create the thread using POSIX routines.
    pthread_attr_t  attr;
    pthread_t       posixThreadID;
    int             returnVal;
    
    // init thread attributes
    returnVal = pthread_attr_init(&attr);
    assert(!returnVal);
    returnVal = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    assert(!returnVal);
    
    //    int threadError = pthread_create(&posixThreadID, &attr, &PosixThreadMainRoutine, NULL);
    int threadError = pthread_create(&posixThreadID, &attr, &test3_thread, NULL);
    
    working = true;
    while(working) {
        //print status and delay at each loop
        printf("*****MAIN-LOOP*****");
        firmataclient_updatestatus(&board);
        printf("*******************");
        //delay(1000);
        int c, d, maxcount=327; //32767;
        for ( c = 1 ; c <= maxcount ; c++ )
            for ( d = 1 ; d <= maxcount ; d++ )
            {}
        
    }
    
    // finish and close thread and port
    returnVal = pthread_attr_destroy(&attr);
    assert(!returnVal);
    if (threadError != 0)
    {
        // Report an error.
    }
    
    printf("Finished all\n");
}

void delayme() {
    for(long j=0; j<2000000000; ++j);
}

void test4() {
    
    int max_ports = MAX_PORTS;
    char *list[max_ports];
    
    // run infinite loop to listen to board for incoming updates
    firmataclient_launchtlisteninghread(&board);
    
    
    // printout list of availble ports
    int num_ports = serial_portlist(list, max_ports);
    for (int i=0; i<num_ports; ++i)
        printf("(%d). %s\n", i+1, list[i]);
    
    // start firmata client
    char *name = "/dev/cu.usbmodem1411";
    int baud = DEFAUTLT_BAUD;
    
    const int len_pins = 4;
    int pins[] = {9, 11, 14, 18};
    int modes[] = {MODE_SERVO, MODE_OUTPUT, MODE_ANALOG, MODE_ANALOG};
    
    // open connection to the board, set requested configuration
    // and request REPORT_FIRMATA
    initConfigPort(name, baud, len_pins, pins, modes);
    
    
    
    for (int k=0; k<1; ++k) {//5
        
        // send new (servo, led) values on board
        firmataclient_setdigitalvalue(&board, 11, 1);
        delayme();
        
        for(int i=0; i<180; i+=30) {
            moveServo(&board, 9, i);
            delayme();
        }
        
        moveServo(&board, 9, 0);
        delayme();
        firmataclient_setdigitalvalue(&board, 11, 0);
        delayme();
    }
    
    // destroy the listening thread
    //void firmataclient_destroylisteningthread();
    // close the port
    firmataclient_closeport(&board);
    printf("Finished all\n");
}

int test5() {
    return 0;
}


int main(int argc, const char * argv[]) {
    
    test5();
    //test4();
    //test2();
    
    return 0;
}

